span.state{
    font-size : 56%;
}

/******************************************************************************/
/* tickets'priority                                                           */
/******************************************************************************/

td.label {
    padding-right: 10px;
}
td.minor, td.normal, td.important {
    height: 16px;
    padding-left: 18px !important;
}

.minor {
    padding: 1px 0px 1px 18px;
    background: url("priority_minor.png") 0% 0.2em no-repeat !important;
}
.normal {
    padding: 1px 0px 1px 18px;
    background: url("priority_normal.png") 0% 0.2em no-repeat !important;
}
.important {
    padding: 1px 0px 1px 18px;
    background: url("priority_important.png") 0% 0.2em no-repeat !important;
}
