
td.alt0 {
    background: #fff;
}
td.alt1 {
    background: #f5deb3; /*wheat*/
}
tr.alt0 {
    background: #f5deb3;
}
tr.alt1 {
    background: #f5deb3;
}

table.stats {
    margin-left: auto;
    margin-right: auto;
}
